import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import {
  isPluginActiveSelector,
  isPluginLoadingSelector,
  isPluginSelectedSelector,
} from '@/redux/plugins/plugins.selectors';
import { removePlugin } from '@/redux/plugins/plugins.slice';
import { PluginsManager } from '@/services/pluginsManager';
import axios from 'axios';

type UseLoadPluginReturnType = {
  togglePlugin: () => void;
  loadPlugin: () => Promise<void>;
  unloadPlugin: () => void;
  reloadPlugin: () => void;
  isPluginActive: boolean;
  isPluginLoading: boolean;
  isPluginSelected: boolean;
};

type UseLoadPluginProps = {
  hash: string;
  pluginUrl: string;
};

export const useLoadPlugin = ({ hash, pluginUrl }: UseLoadPluginProps): UseLoadPluginReturnType => {
  const isPluginActive = useAppSelector(state => isPluginActiveSelector(state, hash));
  const isPluginLoading = useAppSelector(state => isPluginLoadingSelector(state, hash));
  const isPluginSelected = useAppSelector(state => isPluginSelectedSelector(state, hash));

  const dispatch = useAppDispatch();

  const handleLoadPlugin = async (): Promise<void> => {
    const response = await axios(pluginUrl);
    const pluginScript = response.data;

    /* eslint-disable no-new-func */
    const loadPlugin = new Function(pluginScript);

    PluginsManager.setHashedPlugin({
      pluginUrl,
      pluginScript,
    });

    loadPlugin();
  };

  const handleUnloadPlugin = (): void => {
    dispatch(removePlugin({ pluginId: hash }));
    PluginsManager.removePluginContent({ hash });
  };

  const handleReloadPlugin = async (): Promise<void> => {
    handleUnloadPlugin();
    await handleLoadPlugin();
  };

  const togglePlugin = (): void => {
    if (isPluginActive) {
      handleUnloadPlugin();
    } else {
      handleLoadPlugin();
    }
  };

  return {
    isPluginSelected,
    togglePlugin,
    loadPlugin: handleLoadPlugin,
    unloadPlugin: handleUnloadPlugin,
    reloadPlugin: handleReloadPlugin,
    isPluginActive,
    isPluginLoading,
  };
};
