/* eslint-disable no-magic-numbers */
import { fireEvent, render, screen } from '@testing-library/react';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { InitialStoreState } from '@/utils/testing/getReduxStoreActionsListener';
import { StoreType } from '@/redux/store';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import MockAdapter from 'axios-mock-adapter';
import axios, { HttpStatusCode } from 'axios';
import { apiPath } from '@/redux/apiPath';
import { pluginFixture } from '@/models/fixtures/pluginFixture';
import { act } from 'react-dom/test-utils';
import { LoadPluginFromUrl } from './LoadPluginFromUrl.component';

const mockedAxiosApiClient = mockNetworkResponse();
const mockedAxiosClient = new MockAdapter(axios);

const renderComponent = (initialStore?: InitialStoreState): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStore);
  return (
    render(
      <Wrapper>
        <LoadPluginFromUrl />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('LoadPluginFromUrl - component', () => {
  describe('when always', () => {
    it('renders plugin input label', () => {
      renderComponent();

      const pluginInputLabel = screen.getByLabelText('URL:');
      expect(pluginInputLabel).toBeInTheDocument();
    });

    it('renders plugin input', () => {
      renderComponent();

      const pluginInput = screen.getByTestId('load-plugin-input-url');
      expect(pluginInput).toBeInTheDocument();
    });

    it('renders plugin load button', () => {
      renderComponent();

      const loadButton = screen.getByText('Load');
      expect(loadButton.tagName).toBe('BUTTON');
      expect(loadButton).toBeInTheDocument();
    });
    it('should unload plugin after click', async () => {
      mockedAxiosApiClient.onPost(apiPath.registerPluign()).reply(HttpStatusCode.Ok, pluginFixture);
      mockedAxiosApiClient
        .onGet(apiPath.getPlugin(pluginFixture.hash))
        .reply(HttpStatusCode.Ok, pluginFixture);
      mockedAxiosClient.onGet(pluginFixture.urls[0]).reply(HttpStatusCode.Ok, '');

      renderComponent();
      const input = screen.getByTestId('load-plugin-input-url');
      expect(input).toBeVisible();

      await act(() => {
        fireEvent.change(input, { target: { value: pluginFixture.urls[0] } });
      });

      expect(input).toHaveValue(pluginFixture.urls[0]);

      const button = screen.queryByTestId('load-plugin-button');
      expect(button).toBeVisible();

      await act(() => {
        button?.click();
      });

      expect(button).toBeDisabled();
    });
  });
});
